from __future__ import annotations

from typing import Any, Callable, Dict, List, Optional, Type, Union, Sequence, Tuple

import click
from click.core import Context, Parameter
from typer.core import TyperCommand, TyperGroup

from .formatting import HelpFormatter


def _rich_typer_format_banner(
    self: click.core.Command,
    ctx: Context,
    formatter: HelpFormatter
) -> None:
    if self.banner:
        formatter.write_banner(self.banner, self.banner_justify)


def _rich_typer_format_options(
    self: click.core.Command,
    ctx: Context,
    formatter: HelpFormatter
) -> None:
    args = []
    opts = []
    for param in self.get_params(ctx):
        rv = param.get_help_record(ctx)
        if rv is not None:
            if param.param_type_name == "argument":
                args.append(rv)
            elif param.param_type_name == "option":
                opts.append(rv)

    if args:
        with formatter.section("Arguments") as table:
            formatter.add_params(args, table)
    if opts:
        with formatter.section("Options") as table:
            formatter.add_params(opts, table)


class RichContext(click.core.Context):
    formatter_class: Type["HelpFormatter"] = HelpFormatter


class RichCommand(TyperCommand):
    context_class: Type["Context"] = RichContext

    def __init__(
        self,
        name: Optional[str],
        context_settings: Optional[Dict[str, Any]] = None,
        callback: Optional[Callable[..., Any]] = None,
        params: Optional[List["Parameter"]] = None,
        help: Optional[str] = None,
        epilog: Optional[str] = None,
        epilog_blend: Optional[Tuple[Tuple[int, int, int],
                                     Tuple[int, int, int]]] = None,
        short_help: Optional[str] = None,
        banner: Optional[str] = None,
        banner_justify: Optional[str] = 'default',
        usage: Optional[str] = None,
        options_metavar: Optional[str] = "[OPTIONS]",
        add_help_option: bool = True,
        no_args_is_help: bool = False,
        hidden: bool = False,
        deprecated: bool = False,
    ) -> None:
        self.banner = banner
        self.banner_justify = banner_justify
        self.epilog_blend = epilog_blend
        self.usage = usage
        super().__init__(
            name=name,
            context_settings=context_settings,
            callback=callback,
            params=params,
            help=help,
            epilog=epilog,
            short_help=short_help,
            options_metavar=options_metavar,
            add_help_option=add_help_option,
            no_args_is_help=no_args_is_help,
            hidden=hidden,
            deprecated=deprecated
        )

    def format_help(self, ctx: "Context", formatter: HelpFormatter) -> None:
        self.format_banner(ctx, formatter)
        self.format_usage(ctx, formatter)
        self.format_help_text(ctx, formatter)
        self.format_options(ctx, formatter)
        self.format_epilog(ctx, formatter)

    def format_banner(self, ctx: "Context", formatter: HelpFormatter) -> None:
        _rich_typer_format_banner(self, ctx=ctx, formatter=formatter)

    def format_usage(self, ctx: "Context", formatter: HelpFormatter) -> None:
        if self.usage:
            formatter.write(self.usage)
            formatter.write("\n")
        else:
            super().format_usage(ctx, formatter)

    def format_options(self, ctx: "Context", formatter: HelpFormatter) -> None:
        _rich_typer_format_options(self, ctx=ctx, formatter=formatter)

    def format_epilog(self, ctx: "Context", formatter: HelpFormatter) -> None:
        if self.epilog:
            formatter.write_epilog(self.epilog, self.epilog_blend)


class RichGroup(TyperGroup):
    context_class: Type["Context"] = RichContext

    def __init__(
        self,
        name: Optional[str] = None,
        commands: Optional[Union[Dict[str, RichCommand],
                                 Sequence[RichCommand]]] = None,
        **attrs: Any,
    ) -> None:
        self.banner = attrs.pop("banner", None)
        self.banner_justify = attrs.pop("banner_justify", "default")
        self.epilog_blend = attrs.pop("epilog_blend", None)
        self.usage = attrs.pop("usage", None)
        super().__init__(name=name, commands=commands, **attrs)

    def format_help(self, ctx: "Context", formatter: HelpFormatter) -> None:
        self.format_banner(ctx, formatter)
        self.format_usage(ctx, formatter)
        self.format_help_text(ctx, formatter)
        self.format_options(ctx, formatter)
        self.format_epilog(ctx, formatter)

    def format_banner(self, ctx: "Context", formatter: HelpFormatter) -> None:
        _rich_typer_format_banner(self, ctx, formatter)

    def format_usage(self, ctx: "Context", formatter: HelpFormatter) -> None:
        if self.usage:
            formatter.write(self.usage)
            formatter.write("\n")
        else:
            super().format_usage(ctx, formatter)

    def format_commands(self, ctx: Context, formatter: HelpFormatter) -> None:
        """Extra format methods for multi methods that adds all the commands
        after the options.
        """
        commands = []
        for subcommand in self.list_commands(ctx):
            cmd = self.get_command(ctx, subcommand)
            # What is this, the tool lied about a command.  Ignore it
            if cmd is None:
                continue
            if cmd.hidden:
                continue

            commands.append((subcommand, cmd))

        # allow for 3 times the default spacing
        if len(commands):
            limit = formatter.width - 6 - max(len(cmd[0]) for cmd in commands)

            rows = []
            for subcommand, cmd in commands:
                help = cmd.get_short_help_str(limit)
                rows.append((subcommand, help))

            if rows:
                with formatter.section("Commands") as table:
                    formatter.add_params(rows, table)

    def format_options(self, ctx: "Context", formatter: HelpFormatter) -> None:
        _rich_typer_format_options(self, ctx=ctx, formatter=formatter)
        self.format_commands(ctx, formatter)

    def format_epilog(self, ctx: "Context", formatter: HelpFormatter) -> None:
        if self.epilog:
            formatter.write_epilog(self.epilog, self.epilog_blend)



from __future__ import annotations

import re
from contextlib import contextmanager
from typing import Iterator, List, Optional, Tuple, Dict

from click import HelpFormatter as ClickHelpFormatter
from rich.console import Console, JustifyMethod
from rich.highlighter import RegexHighlighter
from rich.panel import Panel
from rich.table import Table
from rich.text import Text
from rich.theme import Theme

from .utils import blend_text


class HelpFormatter(ClickHelpFormatter):

    def __init__(
        self,
        indent_increment: int = 2,
        width: Optional[int] = None,
        max_width: Optional[int] = None,
    ) -> None:
        super().__init__(indent_increment=indent_increment, width=width, max_width=max_width)
        self.highlighters = self.init_highlighters()
        self.console = self.init_console()

    def init_highlighters(self) -> Dict[str, RegexHighlighter]:
        class OptionHighlighter(RegexHighlighter):
            highlights = [
                r"(?P<switch>^\-\w$)",
                r"(?P<option>^\-\-[\w\-]+)(?P<metavar>\s.*$)?",
                r"(?P<args_and_cmds>^[\w]+$)",
            ]

        class HelpHighlighter(RegexHighlighter):
            highlights = [
                # 匹配最后一个小括号，且小括号前面有空格
                r"(?P<help_require>(\()(?!.*\2)(?<=\s\2)(.+)\)$)",
            ]

        return_highlighters = {
            "opt": OptionHighlighter(), "help": HelpHighlighter()}

        return return_highlighters

    def init_console(self) -> Console:
        console = Console(
            theme=Theme(
                {
                    "option": "bold cyan",
                    "switch": "bold green",
                    "metavar": "bold yellow",
                    "help_require": "dim",
                    "args_and_cmds": "yellow"
                }
            ),
            # highlighter=self.highlighter,
        )
        return console

    @contextmanager
    def section(self, name: str) -> Iterator[None]:
        options_table = Table(highlight=True, box=None, show_header=False)
        yield options_table
        self.write(Panel(
            options_table, border_style="dim", title=name, title_align="left"
        ))

    def add_params(self, params: List[Tuple[str, str]], table: Table) -> None:
        for name, help in params:
            arg_list = name.split(',')
            if len(arg_list) == 2:
                opt1 = self.highlighters['opt'](arg_list[0])
                opt2 = self.highlighters['opt'](arg_list[1].strip())
            else:
                opt1 = Text("")
                opt2 = self.highlighters['opt'](arg_list[0])
            help = self.escape_text(help)
            help = Text.from_markup(help, emoji=False)

            table.add_row(opt1, opt2, self.highlighters['help'](help))

    def escape_text(self, text: str) -> str:
        match = re.search(r"(\[)(?!.*\1)(?<=\s\1)(.+)\]$",
                          text)  # 匹配最后一个中括号，且中括号前面有空格
        if match:
            text = text.replace("[%s]" % match.group(2),
                                "(%s)" % match.group(2))
        return text

    def write_usage(
        self, prog: str, args: str = "", prefix: Optional[str] = None
    ) -> None:
        # ! 如果过长 会导致换行
        if not prefix:
            prefix = "[dim]Usage: [/]"

        usage_prefix = f"{prefix}{prog}"

        table = Table(highlight=True, box=None, show_header=False)
        table.add_row(usage_prefix, f"[yellow]{args}[/]")
        self.write(table)
        self.write("\n")

    def write_banner(self, banner: str, justify: Optional[JustifyMethod] = 'default') -> None:
        self.write(Text.from_markup(banner, emoji=False), justify)

    def write_epilog(
        self, epilog: str,
        blend: Optional[Tuple[Tuple[int, int, int],
                              Tuple[int, int, int]]] = None
    ) -> None:
        if blend is False:
            self.write(epilog, "right")
            return

        if blend is None:
            blend = ((32, 32, 255), (255, 32, 255))
        self.write(blend_text(epilog, blend), "right")

    def write(
        self, string: str | Text | Panel,
        justify: Optional[JustifyMethod] = None
    ) -> None:
        if string == "\n":
            self.console.print()
        else:
            self.console.print(string, justify=justify)



from .main import RichTyper
from . import Argument, Option, __version__


app = RichTyper()
banner = f"[b]Rich Typer[/b] [magenta]v{__version__}[/] 🤑\n\n[dim]将 Rich 与 Typer 结合起来，使界面更加漂亮。\n"

url = "♥ https://github.com/Elinpf/rich_typer"


@app.command(banner=banner, banner_justify='center', epilog=url)
def main(
    name: str = Argument(...,
                         help="Name of the [green]person to greet[/]."),
    message: str = Option('ms', '-m', '--message',
                                help="The message [red]to[/] display"),
    version: bool = Option(False, '-v', '--version',
                           help="Show the [u]version[/] and exit"),
) -> None:
    """[bold][blue]Rich Typer[/] example."""
    ...


app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rich_typer']

package_data = \
{'': ['*']}

install_requires = \
['rich>=11.2.0,<12.0.0', 'typer>=0.4.0,<0.5.0']

setup_kwargs = {
    'name': 'rich-typer',
    'version': '0.1.1',
    'description': 'more beautiful Typer',
    'long_description': '# Rich Typer\n\n## 介绍\n\n本项目是将Rich和Typer结合在一起，将CLI打造的更加美观漂亮。\n\n![](images/example.png)\n\n## 安装\n\n```bash\npip install rich_typer\n```\n\n或者使用`poetry` 安装\n\n```bash\ngit clone https://github.com/Elinpf/rich_typer\ncd rich_typer\npoetry build\npip install dist/<whl_file>\n```\n\n## 使用\n\n完全兼容[Typer](https://github.com/tiangolo/typer)语法，具体语法细节参考[Typer官方文档](https://typer.tiangolo.com/)\n\n\n除此之外增加了如下几个参数：\n\n- `banner`  增加标题\n- `banner_justify` 标题位置\n- `epilog_blend` 底部信息的渐变色\n- `usage` 自定义Usage\n\n## Example\n\n```py\nfrom rich_typer import RichTyper, Argument, Option\n\n\napp = RichTyper()\nbanner = f"[b]Rich Typer[/b] [magenta][/] 🤑\\n\\n[dim]将 Rich 与 Typer 结合起来，使界面更加漂亮。\\n"\n\nurl = "♥ https://github.com/Elinpf/rich_typer"\n\n\n@app.command(banner=banner, banner_justify=\'center\', epilog=url)\ndef main(\n    name: str = Argument(...,\n                         help="Name of the [green]person to greet[/]."),\n    message: str = Option(\'ms\', \'-m\', \'--message\',\n                                help="The message [red]to[/] display"),\n    version: bool = Option(False, \'-v\', \'--version\',\n                           help="Show the [u]version[/] and exit"),\n) -> None:\n    """[bold][blue]Rich Typer[/] example."""\n    ...\n\n\napp()\n```\n',
    'author': 'Elin',
    'author_email': '365433079@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Elinpf/rich_typer',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
